/*
 *  SPDX-License-Identifier: Apache-2.0
 *
 *  Copyright The original authors
 *
 *  Licensed under the Apache Software License version 2.0, available at http://www.apache.org/licenses/LICENSE-2.0
 */
package org.kcctl.command;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;

import jakarta.inject.Inject;

import org.eclipse.microprofile.rest.client.RestClientBuilder;
import org.kcctl.completion.ConnectorNameCompletions;
import org.kcctl.service.KafkaConnectApi;
import org.kcctl.util.ConfigurationContext;
import org.kcctl.util.Connectors;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import picocli.CommandLine;
import picocli.CommandLine.Command;
import picocli.CommandLine.Model.CommandSpec;
import picocli.CommandLine.Option;
import picocli.CommandLine.ParameterException;
import picocli.CommandLine.Parameters;
import picocli.CommandLine.Spec;

@Command(name = "connector", aliases = "connectors", description = "Patches the specified connector(s) with the given configuration parameters")
public class PatchConnectorCommand implements Callable<Integer> {

    @CommandLine.Mixin
    HelpMixin help;

    @Spec
    CommandSpec commandSpec;

    @Parameters(paramLabel = "CONNECTOR NAME", description = "Name of the connector", completionCandidates = ConnectorNameCompletions.class)
    Set<String> names = Set.of();

    @Option(names = { "-e", "--reg-exp" }, description = "use CONNECTOR NAME(s) as regexp pattern(s) to use on all connectors")
    boolean regexpMode = false;

    @Option(names = { "-s", "--set" }, description = "Set the following configuration parameter")
    Map<String, String> setParameters;

    @Option(names = { "-r", "--remove" }, description = "Remove the following configuration parameter")
    List<String> removeParameters;

    private final ConfigurationContext context;

    @Inject
    public PatchConnectorCommand(ConfigurationContext context) {
        this.context = context;
    }

    // Hack : Picocli currently require an empty constructor to generate the completion file
    public PatchConnectorCommand() {
        context = new ConfigurationContext();
    }

    @Override
    public Integer call() throws JsonProcessingException, InterruptedException, ExecutionException {

        KafkaConnectApi kafkaConnectApi = RestClientBuilder.newBuilder()
                .baseUri(context.getCurrentContext().getCluster())
                .build(KafkaConnectApi.class);

        Set<String> selectedConnector = Connectors.getSelectedConnectors(kafkaConnectApi, names, regexpMode);
        for (String connectorToPatch : selectedConnector) {
            int returnCode = patch(kafkaConnectApi, connectorToPatch);
            if (returnCode > 0)
                return returnCode;
        }

        return 0;
    }

    private int patch(KafkaConnectApi kafkaConnectApi, String conectorToPatch) throws JsonProcessingException {
        Map<String, String> connectorParameters = kafkaConnectApi.getConnectorConfig(conectorToPatch);

        if (setParameters == null && removeParameters == null) {

            throw new ParameterException(commandSpec.commandLine(), "Missing required arguments: " +
                    " please enter at least one parameter to set or remove");
        }

        if (setParameters != null) {
            connectorParameters.putAll(setParameters);
        }

        if (removeParameters != null) {
            removeParameters.forEach(connectorParameters::remove);
        }

        String connectorParametersString = new ObjectMapper().writeValueAsString(connectorParameters);
        kafkaConnectApi.updateConnector(conectorToPatch, connectorParametersString);

        DescribeConnectorCommand describeConnectorCommand = new DescribeConnectorCommand(context);
        describeConnectorCommand.names = Set.of(conectorToPatch);
        describeConnectorCommand.includeTasksConfig = false;

        System.out.println("New connector configuration:");
        describeConnectorCommand.call();

        return 0;

    }
}
