/*
 *  SPDX-License-Identifier: Apache-2.0
 *
 *  Copyright The original authors
 *
 *  Licensed under the Apache Software License version 2.0, available at http://www.apache.org/licenses/LICENSE-2.0
 */
package org.kcctl.command;

import java.io.IOException;
import java.time.Duration;

import org.junit.jupiter.api.DisplayNameGeneration;
import org.junit.jupiter.api.DisplayNameGenerator;
import org.junit.jupiter.api.Test;
import org.kcctl.IntegrationTest;
import org.kcctl.IntegrationTestProfile;
import org.kcctl.service.ConnectorOffsets;
import org.kcctl.support.InjectCommandContext;
import org.kcctl.support.KcctlCommandContext;
import org.kcctl.support.SkipIfConnectVersionIsOlderThan;
import org.testcontainers.shaded.com.fasterxml.jackson.databind.ObjectMapper;

import io.debezium.testing.testcontainers.Connector;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.TestProfile;
import picocli.CommandLine;

import static org.assertj.core.api.Assertions.assertThat;
import static org.awaitility.Awaitility.await;

@QuarkusTest
@TestProfile(IntegrationTestProfile.class)
@DisplayNameGeneration(DisplayNameGenerator.ReplaceUnderscores.class)
@SkipIfConnectVersionIsOlderThan("3.6")
class DeleteOffsetsCommandTest extends IntegrationTest {

    private static final long OFFSET_AVAILABILITY_TIMEOUT_SECONDS = 30;

    private final ObjectMapper mapper = new ObjectMapper();

    @InjectCommandContext
    KcctlCommandContext<DeleteOffsetsCommand> deleteContext;

    @InjectCommandContext
    KcctlCommandContext<GetOffsetsCommand> getContext;

    @InjectCommandContext
    KcctlCommandContext<StopConnectorCommand> stopContext;

    @Test
    public void delete_offsets_of_running_connector_should_return_error() {

        registerTestConnector("delete-offsets-test1");

        await()
                .atMost(Duration.ofSeconds(OFFSET_AVAILABILITY_TIMEOUT_SECONDS))
                .until(() -> {
                    deleteContext.reset();

                    // No offsets have been committed yet; wait a little longer for an offset commit to complete
                    return isOffsetAvailable("delete-offsets-test1");
                });

        int exitCode = deleteContext.commandLine().execute("delete-offsets-test1");
        String errorMessage = deleteContext.error().toString();

        assertThat(exitCode).isEqualTo(CommandLine.ExitCode.SOFTWARE);
        assertThat(errorMessage).contains(
                "Connectors must be in the STOPPED state before their offsets can be modified. This can be done for the specified connector by issuing a 'PUT' request to the '/connectors/delete-offsets-test1/stop' endpoint");
    }

    @Test
    public void offsets_are_correctly_deleted() {

        registerTestConnector("delete-offsets-test1");

        await()
                .atMost(Duration.ofSeconds(OFFSET_AVAILABILITY_TIMEOUT_SECONDS))
                .until(() -> {
                    deleteContext.reset();

                    // No offsets have been committed yet; wait a little longer for an offset commit to complete
                    return isOffsetAvailable("delete-offsets-test1");
                });

        stopContext.runAndEnsureExitCodeOk("delete-offsets-test1");
        kafkaConnect.ensureConnectorState("delete-offsets-test1", Connector.State.STOPPED);

        int exitCode = deleteContext.commandLine().execute("delete-offsets-test1");
        String output = deleteContext.output().toString();

        assertThat(exitCode).isEqualTo(CommandLine.ExitCode.OK);
        assertThat(output).isEqualTo("The offsets for this connector have been reset successfully\n");
    }

    @Test
    public void offsets_are_correctly_deleted_multiple_connectors() {

        registerTestConnectors("delete-offsets-test1", "delete-offsets-test2");

        await()
                .atMost(Duration.ofSeconds(OFFSET_AVAILABILITY_TIMEOUT_SECONDS))
                .until(() -> {
                    deleteContext.reset();

                    // No offsets have been committed yet; wait a little longer for an offset commit to complete
                    return isOffsetAvailable("delete-offsets-test1") || isOffsetAvailable("delete-offsets-test2");
                });

        stopContext.runAndEnsureExitCodeOk("delete-offsets-test1", "delete-offsets-test2");
        kafkaConnect.ensureConnectorState("delete-offsets-test1", Connector.State.STOPPED);
        kafkaConnect.ensureConnectorState("delete-offsets-test2", Connector.State.STOPPED);

        int exitCode = deleteContext.commandLine().execute("delete-offsets-test1", "delete-offsets-test2");
        String output = deleteContext.output().toString();

        assertThat(exitCode).isEqualTo(CommandLine.ExitCode.OK);
        assertThat(output).isEqualTo("""
                The offsets for this connector have been reset successfully
                The offsets for this connector have been reset successfully
                """);
    }

    private boolean isOffsetAvailable(String connectorName) throws IOException {

        getContext.reset();
        getContext.runAndEnsureExitCodeOk(connectorName);
        String output = getContext.output().toString();
        ConnectorOffsets offsets = mapper.readValue(output, ConnectorOffsets.class);

        return !offsets.offsets().isEmpty();
    }

}
